package com.cmb.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.cmb.service.HclcCmbPolyService;
import com.cmb.util.MD5Utils;
import com.cmb.util.SM2Util;
import com.cmb.util.SignatureUtil;
import com.cmb.util.Utils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.core.IRequest;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

@Service
public class HclcCmbPolyServiceImpl implements HclcCmbPolyService {
    private static final String url = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/qrcodeapply";  //UAT
    //uat环境商户国密私钥
    public static final String privateKey = "D5F2AFA24E6BA9071B54A8C9AD735F9A1DE9C4657FA386C09B592694BC118B38";
    //uat环境招行国密公钥
    public static final String publicKey = "MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAE6Q+fktsnY9OFP+LpSR5Udbxf5zHCFO0PmOKlFNTxDIGl8jsPbbB/9ET23NV+acSz4FEkzD74sW2iiNVHRLiKHg==";

    public static final String appId = "8ab74856-8772-45c9-96db-54cb30ab9f74";
    public static final String appSecret = "5b96f20a-011f-4254-8be8-9a5ceb2f317f";

    @Value("${cmb.version}")
    private String version;
    @Value("${cmb.encoding}")
    private String encoding;
    @Value("${cmb.signMethod}")
    private String signMethod;

    /*币种，非必传，默认156人民币*/
    @Value("${cmb.currencyCode}")
    private String currencyCode;

    @Override
    public JSONObject getQrcode(IRequest iRequest, JSONObject params) {
        JSONObject result = new JSONObject();

        Map<String, String> requestPublicParams = JSONObject.parseObject(params.toJSONString(),Map.class);
        // 组装requestBody并加签
        String signResult= signMethod(requestPublicParams);
        try {
            ObjectMapper mapper = new ObjectMapper();
            Map<String,String> signResultMap = mapper.readValue(signResult, Map.class);
            long currentTimeMills = System.currentTimeMillis() / 1000;

            // 组apiSign加密Map
            Map<String,String> apiSign = new TreeMap<>();
            apiSign.put("appid", appId);
            apiSign.put("secret", appSecret);
            apiSign.put("sign", signResultMap.get("sign"));
            apiSign.put("timestamp", "" + currentTimeMills);

            // MD5加密
            String MD5Content = SignatureUtil.getSignContent(apiSign);
            String apiSignString = MD5Utils.getMD5Content(MD5Content).toLowerCase();

            // 组request头部Map
            Map<String, String> apiHeader = new HashMap<>();
            apiHeader.put("appid", appId);
            apiHeader.put("timestamp", "" + currentTimeMills);
            apiHeader.put("apisign", apiSignString);

            // 发送HTTP post请求
            Map<String,String> response = Utils.postForEntity(url, signResult, apiHeader);
/*
            版本号	    version	String(5)	    是	固定为0.0.1
            编码方式	    encoding	String(20)	是	固定为UTF-8
            签名	sign	String(1024)	        是
            签名方法    	signMethod	String(2)	是	02：SM2
            返回码	    returnCode	String(7)	是	SUCCESS/FAIL,此字段是通信标识，非交易标识，交易是否成功需要查看respCode来判断。SUCCESS表示商户上送的报文符合规范，FAIL表示报文内的字段不符合规范，包括长度超长、非法字符、签名错误等
                    以下字段仅在returnCode为SUCCESS时返回
            商户号	    merId	String(32)	    是	请求报文中的商户号
            商户订单号   	orderId	String(32)	    是	请求报文中的商户订单号
            响应码 	    respCode	String(10)	是	业务错误码，成功为SUCCESS，失败为FAIL
                    以下字段仅在returnCode和respCode为SUCCESS时返回
            二维码	    qrCode	String(300)	    是
            平台订单号	cmbOrderId	String(32)	是	招行生成的订单号
            订单发送时间	txnTime	String(14)	    是	订单生成时间，格式为yyyyMMddHHmmss
                    以下字段仅在returnCode或respCode为FAIL时返回
            错误码	    errCode	String(32)	    是	请求处理失败的错误码信息
            应答信息	    respMsg	String(256)	    是	请求处理失败的详细描述信息
*/

            System.out.println(mapper.writeValueAsString(response));

            // 返回结果验签,验证数据的正确性（安全性）
            Boolean checkResult1 = checkSign(mapper.writeValueAsString(response));
            if(!checkResult1){
                //数据验签失败的处理，重发或者直接告诉前端发送失败
                result.put("errCode",response.get("errCode"));
            }
            //获取响应结果内容
            String success = response.get("returnCode");
            System.out.println("返回结果：" + success);
            result.put("qrcode",response.get("qrcode"));
            result.put("respMsg",response.get("respMsg"));
            //业务数据的封装。。。
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            result.put("errCode","服务器问题");
            return result;
        }
    }

    private String signMethod(Map paramsMap){
        Map<String, String> requestPublicParams = new TreeMap<>();
        String requestStr = null;
        try {
            //公共请求参数
            requestPublicParams.put("version", version);    //版本号，固定为0.0.1(必传字段)
            requestPublicParams.put("encoding", encoding);   //编码方式，固定为UTF-8(必传)
            requestPublicParams.put("signMethod", signMethod);    //签名方法，固定为02，表示签名方式为国密(必传)
//            //业务要素
//            Map<String, String> requestTransactionParams = new HashMap<>();
//            requestTransactionParams.put("body", "聚合支付测试");   //商户号(必传)
//            requestTransactionParams.put("currencyCode", "156");    //交易币种，默认156，目前只支持人民币（156）
//            requestTransactionParams.put("merId", "3089991074200KV");   //商户号(必传)
//            requestTransactionParams.put("notifyUrl", notifyUrl);  //交易通知地址(必传)
//            requestTransactionParams.put("orderId", "" + System.currentTimeMillis()); //商户订单号(必传)
//            requestTransactionParams.put("payValidTime", "1200"); //支付有效时间
//            requestTransactionParams.put("termId", "00774411");  //终端号
//            requestTransactionParams.put("txnAmt", "1405");  //交易金额,单位为分(必传)
//            requestTransactionParams.put("userId", "N003363806");   //收银员
//            requestTransactionParams.put("tradeScene", "OFFLINE");   //交易场景
            ObjectMapper mapper = new ObjectMapper();
            requestPublicParams.put("biz_content", mapper.writeValueAsString(paramsMap));

            System.out.println("加签前的报文内容：" + mapper.writeValueAsString(requestPublicParams));

            //对待加签内容进行排序拼接
            String signContent= SignatureUtil.getSignContent(requestPublicParams);
            //加签
            requestPublicParams.put("sign", SM2Util.sm2Sign(signContent, privateKey));

            requestStr = mapper.writeValueAsString(requestPublicParams);

            System.out.println("加签后的报文内容：" + requestStr);
            return requestStr;

        }catch (Exception e){
            System.out.println("加签发生异常！");
            e.printStackTrace();
            return requestStr;
        }
    }

    private Boolean checkSign(String string){
        System.out.println("要验签的报文内容：" + string);
        try {
            //验签
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, String> responseBodyMap = objectMapper.readValue(string, Map.class);
            String sign = responseBodyMap.remove("sign");
            String contentStr = SignatureUtil.getSignContent(responseBodyMap);
            boolean result = SM2Util.sm2Check(contentStr,sign, publicKey);

            if (result) {
                System.out.println("报文验签成功!");
            } else {
                System.out.println("报文验签失败!");
            }
            return result;
        }catch (Exception e){
            System.out.println("验签发生异常！");
            e.printStackTrace();
            return false;
        }
    }
}
